# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import select


# TODO: unskip the doctest on line 1373 after making work on Python 3
class TaleaRhythmMaker(RhythmMaker):
    r'''Talea rhythm-maker.

    ..  container:: example

        Repeats talea of 1/16, 2/16, 3/16, 4/16:

        ::

            >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=rhythmmakertools.Talea(
            ...         counts=[1, 2, 3, 4],
            ...         denominator=16,
            ...         ),
            ...     )

        ::

            >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 3/8
                    c'16 [
                    c'8
                    c'8. ]
                }
                {
                    \time 4/8
                    c'4
                    c'16 [
                    c'8
                    c'16 ~ ]
                }
                {
                    \time 3/8
                    c'8
                    c'4
                }
                {
                    \time 4/8
                    c'16 [
                    c'8
                    c'8.
                    c'8 ]
                }
            }

    ..  container:: example

        Formats rhythm-maker:

        ::

            >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=rhythmmakertools.Talea(
            ...         counts=[1, 2, 3, 4],
            ...         denominator=16,
            ...         ),
            ...     )

        ::

            >>> f(rhythm_maker)
            rhythmmakertools.TaleaRhythmMaker(
                talea=rhythmmakertools.Talea(
                    counts=[1, 2, 3, 4],
                    denominator=16,
                    ),
                )

    Follows the two-step configure-once / call-repeatedly pattern shown here.

    Object model of a partially evaluated function that accepts a (possibly
    empty) list of divisions as input and returns a list of selections as
    output (structured one selection per input division).
    '''

    r'''Example helpers:

    # used in a piece with four voices:
    # voice 1 starts reading talea at beginning of talea;
    # voice 2 starts reading talea at second event of talea;
    # voice 3 starts reading talea at third event of talea;
    # voice 4 starts reading talea at fourth event of talea.
    def helper(talea, rotation):
        assert len(rotation) == 2
        if not talea:
            return talea
        voice_index, measure_index = rotation
        talea = sequencetools.Sequence(talea).rotate(n=-voice_index)
        return talea

    # used in a piece with four voices:
    # voice 1 starts reading talea at beginning of talea;
    # voice 2 starts reading talea 1/4 of way through talea;
    # voice 3 starts reading talea 2/4 of way through talea;
    # voice 4 starts reading talea 3/4 of way through talea.
    def quarter_rotation_helper(talea, rotation):
        assert len(rotation) == 2
        if not talea:
            return talea
        voice_index, measure_index = rotation
        index_of_rotation = -voice_index * (len(talea) // 4)
        index_of_rotation += -4 * measure_index
        talea = sequencetools.Sequence(talea).rotate(n=index_of_rotation)
        return talea
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        '_burnish_specifier',
        '_extra_counts_per_division',
        '_helper_functions',
        '_read_talea_once_only',
        '_rest_tied_notes',
        '_split_divisions_by_counts',
        '_talea',
        '_tie_split_notes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        talea=None,
        beam_specifier=None,
        burnish_specifier=None,
        division_masks=None,
        duration_spelling_specifier=None,
        extra_counts_per_division=None,
        logical_tie_masks=None,
        read_talea_once_only=None,
        rest_tied_notes=None,
        split_divisions_by_counts=None,
        tie_specifier=None,
        tie_split_notes=True,
        tuplet_spelling_specifier=None,
        helper_functions=None,
        ):
        from abjad.tools import rhythmmakertools
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            division_masks=division_masks,
            logical_tie_masks=logical_tie_masks,
            tie_specifier=tie_specifier,
            tuplet_spelling_specifier=tuplet_spelling_specifier,
            )
        prototype = (rhythmmakertools.Talea, type(None))
        assert isinstance(talea, prototype)
        assert isinstance(read_talea_once_only, (bool, type(None)))
        self._read_talea_once_only = read_talea_once_only
        self._talea = talea
        if tie_split_notes is not None:
            assert isinstance(tie_split_notes, bool), repr(tie_split_notes)
        self._tie_split_notes = tie_split_notes
        helper_functions = helper_functions or {}
        talea_helper = helper_functions.get('talea')
        extra_counts_per_division_helper = helper_functions.get(
            'extra_counts_per_division')
        lefts_helper = helper_functions.get('left_classes')
        middles_helper = helper_functions.get('middle_classes')
        rights_helper = helper_functions.get('right_classes')
        left_lengths_helper = helper_functions.get('left_counts')
        right_lengths_helper = helper_functions.get('right_counts')
        secondary_divisions_helper = \
            helper_functions.get('split_divisions_by_counts')
        if extra_counts_per_division is not None:
            extra_counts_per_division = tuple(extra_counts_per_division)
        prototype = (rhythmmakertools.BurnishSpecifier, type(None))
        assert isinstance(burnish_specifier, prototype)
        self._burnish_specifier = burnish_specifier
        if split_divisions_by_counts is not None:
            split_divisions_by_counts = tuple(split_divisions_by_counts)
        talea_helper = self._none_to_trivial_helper(talea_helper)
        extra_counts_per_division_helper = self._none_to_trivial_helper(
            extra_counts_per_division_helper)
        lefts_helper = self._none_to_trivial_helper(lefts_helper)
        middles_helper = self._none_to_trivial_helper(middles_helper)
        rights_helper = self._none_to_trivial_helper(rights_helper)
        left_lengths_helper = self._none_to_trivial_helper(
            left_lengths_helper)
        right_lengths_helper = self._none_to_trivial_helper(
            right_lengths_helper)
        secondary_divisions_helper = self._none_to_trivial_helper(
            secondary_divisions_helper)
        assert extra_counts_per_division is None or \
            mathtools.all_are_integer_equivalent_numbers(
                extra_counts_per_division)
        assert split_divisions_by_counts is None or \
            mathtools.all_are_nonnegative_integer_equivalent_numbers(
                split_divisions_by_counts)
        assert callable(talea_helper)
        assert callable(extra_counts_per_division_helper)
        assert callable(lefts_helper)
        assert callable(middles_helper)
        assert callable(rights_helper)
        assert callable(left_lengths_helper)
        assert callable(right_lengths_helper)
        self._extra_counts_per_division = extra_counts_per_division
        self._split_divisions_by_counts = split_divisions_by_counts
        assert isinstance(rest_tied_notes, (bool, type(None))), rest_tied_notes
        self._rest_tied_notes = rest_tied_notes
        if helper_functions == {}:
            helper_functions = None
        self._helper_functions = helper_functions

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls talea rhythm-maker on `divisions`.

        ..  container:: example

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)

            ::

                >>> for selection in selections:
                ...     selection
                Selection([Note("c'16"), Note("c'8"), Note("c'8.")])
                Selection([Note("c'4"), Note("c'16"), Note("c'8"), Note("c'16")])
                Selection([Note("c'8"), Note("c'4")])
                Selection([Note("c'16"), Note("c'8"), Note("c'8."), Note("c'8")])

        ..  todo:: Add rotation examples.

        Returns list of of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            rotation=rotation,
            )

    def __format__(self, format_specification=''):
        r'''Formats talea rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            Formats talea rhythm-maker:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> f(rhythm_maker)
                rhythmmakertools.TaleaRhythmMaker(
                    talea=rhythmmakertools.Talea(
                        counts=[1, 2, 3, 4],
                        denominator=16,
                        ),
                    )

        ..  container:: example

            Storage formats talea rhythm-maker:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> f(rhythm_maker)
                rhythmmakertools.TaleaRhythmMaker(
                    talea=rhythmmakertools.Talea(
                        counts=[1, 2, 3, 4],
                        denominator=16,
                        ),
                    )

        Returns string.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __illustrate__(self, divisions=((3, 8), (4, 8), (3, 16), (4, 16))):
        r'''Illustrates talea rhythm-maker.

        ..  container:: example

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )
                >>> show(rhythm_maker) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = rhythm_maker.__illustrate__()
                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/16
                        c'8 [
                        c'16 ~ ]
                    }
                    {
                        \time 4/16
                        c'8. [
                        c'16 ]
                    }
                }

        Defaults `divisions` to ``3/8``, ``4/8``, ``3/16``, ``4/16``.

        Returns LilyPond file.
        '''
        return RhythmMaker.__illustrate__(self, divisions=divisions)

    def __repr__(self):
        r'''Gets interpreter representation.

        ..  container:: example

            ::

                >>> rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )
                TaleaRhythmMaker(talea=Talea(counts=[1, 2, 3, 4], denominator=16))

        Returns string.
        '''
        return super(TaleaRhythmMaker, self).__repr__()

    ### PRIVATE METHODS ###

    def _apply_burnish_specifier(self, divisions):
        burnish_specifier = self._get_burnish_specifier()
        return burnish_specifier(
            divisions,
            helper_functions=self.helper_functions,
            rotation=self._rotation,
            )

    def _apply_ties_to_split_notes(self, result, unscaled_talea):
        from abjad.tools import rhythmmakertools
        tie_specifier = self._get_tie_specifier()
        if not self.tie_split_notes:
            return
        leaves = select(result).by_leaf()
        written_durations = [leaf.written_duration for leaf in leaves]
        weights = []
        for numerator in unscaled_talea:
            duration = durationtools.Duration(
                numerator,
                self.talea.denominator,
                )
            weight = abs(duration)
            weights.append(weight)
        parts = sequencetools.Sequence(written_durations).partition_by_weights(
            weights=weights,
            allow_part_weights=More,
            cyclic=True,
            overhang=True,
            )
        counts = [len(part) for part in parts]
        parts = sequencetools.Sequence(leaves).partition_by_counts(counts)
        prototype = (spannertools.Tie,)
        for part in parts:
            if any(isinstance(_, scoretools.Rest) for _ in part):
                continue
            part = selectiontools.Selection(part)
            tie_spanner = spannertools.Tie()
            # voodoo to temporarily neuter the contiguity constraint
            tie_spanner._unconstrain_contiguity()
            for component in part:
                # TODO: make top-level detach() work here
                for spanner in component._get_spanners(prototype=prototype):
                    spanner._sever_all_components()
                #detach(prototype, component)
            # TODO: remove usage of Spanner._extend()
            tie_spanner._extend(part)
            tie_spanner._constrain_contiguity()

    def _get_burnish_specifier(self):
        from abjad.tools import rhythmmakertools
        if self.burnish_specifier is not None:
            return self.burnish_specifier
        return rhythmmakertools.BurnishSpecifier()

    def _get_format_specification(self):
        from abjad.tools import systemtools
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if self.tie_split_notes:
            names.remove('tie_split_notes')
        return systemtools.FormatSpecification(
            self,
            storage_format_kwargs_names=names,
            )

    def _get_talea(self):
        from abjad.tools import rhythmmakertools
        if self.talea is not None:
            return self.talea
        return rhythmmakertools.Talea()

    def _handle_rest_tied_notes(self, selections):
        if not self.rest_tied_notes:
            return selections
        # wrap every selection in a temporary container;
        # this allows the call to mutate().replace() to work
        containers = []
        for selection in selections:
            container = scoretools.Container(selection)
            attach('temporary container', container)
            containers.append(container)
        for logical_tie in iterate(selections).by_logical_tie():
            if not logical_tie.is_trivial:
                for note in logical_tie[1:]:
                    rest = scoretools.Rest(note)
                    mutate(note).replace(rest)
                detach(spannertools.Tie, logical_tie.head)
        # remove every temporary container and recreate selections
        new_selections = []
        for container in containers:
            inspector = inspect_(container)
            assert inspector.get_indicator(str) == 'temporary container'
            new_selection = mutate(container).eject_contents()
            new_selections.append(new_selection)
        return new_selections

    def _make_leaf_lists(self, numeric_map, talea_denominator):
        leaf_lists = []
        specifier = self._get_duration_spelling_specifier()
        for map_division in numeric_map:
            leaf_list = scoretools.make_leaves_from_talea(
                map_division,
                talea_denominator,
                decrease_durations_monotonically=\
                    specifier.decrease_durations_monotonically,
                forbidden_written_duration=\
                    specifier.forbidden_written_duration,
                spell_metrically=specifier.spell_metrically,
                )
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _make_music(self, divisions, rotation):
        input_divisions = divisions[:]
        input_ = self._rotate_input(
            helper_functions=self.helper_functions,
            rotation=self._rotation,
            )
        talea = input_['talea']
        extra_counts_per_division = input_['extra_counts_per_division']
        unscaled_talea = tuple(talea)
        split_divisions_by_counts = input_['split_divisions_by_counts']
        taleas = (talea, extra_counts_per_division, split_divisions_by_counts)
        if self.talea is not None:
            talea_denominator = self.talea.denominator
        else:
            talea_denominator = None
        result = self._scale_taleas(divisions, talea_denominator, taleas)
        divisions = result[0]
        lcd = result[1]
        talea = result[2]
        extra_counts_per_division = result[3]
        split_divisions_by_counts = result[4]
        secondary_divisions = self._make_secondary_divisions(
            divisions,
            split_divisions_by_counts,
            )
        if talea:
            numeric_map = self._make_numeric_map(
                secondary_divisions,
                talea,
                extra_counts_per_division,
                )
            leaf_lists = self._make_leaf_lists(numeric_map, lcd)
            if not extra_counts_per_division:
                result = leaf_lists
            else:
                tuplets = self._make_tuplets(secondary_divisions, leaf_lists)
                result = tuplets
            selections = [selectiontools.Selection(x) for x in result]
        else:
            selections = []
            for division in secondary_divisions:
                selection = scoretools.make_leaves([0], [division])
                selections.append(selection)
        beam_specifier = self._get_beam_specifier()
        beam_specifier(selections)
        if talea:
            self._apply_ties_to_split_notes(selections, unscaled_talea)
        selections = self._handle_rest_tied_notes(selections)
        selections = self._apply_division_masks(selections, rotation)
        specifier = self._get_duration_spelling_specifier()
        if specifier.rewrite_meter:
            selections = specifier._rewrite_meter_(
                selections,
                input_divisions,
                )
        return selections

    def _make_numeric_map(self, divisions, talea, extra_counts_per_division):
        assert all(isinstance(_, int) for _ in talea), repr(talea)
        prolated_divisions = self._make_prolated_divisions(
            divisions,
            extra_counts_per_division,
            )
        prolated_divisions = [
            mathtools.NonreducedFraction(_) for _ in prolated_divisions
            ]
        if not talea:
            map_divisions = prolated_divisions
            return map_divisions
        prolated_numerators = [_.numerator for _ in prolated_divisions]
        map_divisions = self._split_sequence_extended_to_weights(
            talea,
            prolated_numerators,
            )
        for list_ in map_divisions:
            assert all(isinstance(_, int) for _ in list_), repr(list_)
        if self.burnish_specifier is not None:
            map_divisions = self._apply_burnish_specifier(map_divisions)
        return map_divisions

    def _make_prolated_divisions(self, divisions, extra_counts_per_division):
        prolated_divisions = []
        for i, division in enumerate(divisions):
            if not extra_counts_per_division:
                prolated_divisions.append(division)
                continue
            prolation_addendum = extra_counts_per_division[i]
            if hasattr(division, 'numerator'):
                numerator = division.numerator
            else:
                numerator = division[0]
            if 0 <= prolation_addendum:
                prolation_addendum %= numerator
            else:
                # NOTE: do not remove the following (nonfunctional) if-else;
                #       preserved for backwards compatability.
                use_old_extra_counts_logic = False
                if use_old_extra_counts_logic:
                    prolation_addendum %= numerator
                else:
                    prolation_addendum %= -numerator
            if isinstance(division, tuple):
                numerator, denominator = division
            else:
                numerator, denominator = division.pair
            prolated_division = (
                numerator + prolation_addendum,
                denominator,
                )
            prolated_divisions.append(prolated_division)
        return prolated_divisions

    def _rotate_input(self, helper_functions=None, rotation=None):
        helper_functions = helper_functions or {}
        if self.talea is not None:
            talea = self.talea.counts or ()
        else:
            talea = ()
        helper_function = helper_functions.get('talea')
        helper_function = self._none_to_trivial_helper(helper_function)
        talea = helper_function(talea, rotation)
        talea = datastructuretools.CyclicTuple(talea)
        extra_counts_per_division = self.extra_counts_per_division or ()
        helper_function = helper_functions.get('extra_counts_per_division')
        helper_function = self._none_to_trivial_helper(helper_function)
        extra_counts_per_division = helper_function(
            extra_counts_per_division,
            rotation,
            )
        extra_counts_per_division = datastructuretools.CyclicTuple(
            extra_counts_per_division)
        split_divisions_by_counts = self.split_divisions_by_counts or ()
        helper_function = helper_functions.get('split_divisions_by_counts')
        helper_function = self._none_to_trivial_helper(helper_function)
        split_divisions_by_counts = helper_function(
            split_divisions_by_counts,
            rotation,
            )
        split_divisions_by_counts = datastructuretools.CyclicTuple(
            split_divisions_by_counts)
        return {
            'extra_counts_per_division': extra_counts_per_division,
            'split_divisions_by_counts': split_divisions_by_counts,
            'talea': talea,
            }

    def _split_sequence_extended_to_weights(self, sequence, weights):
        assert mathtools.all_are_positive_integers(weights), repr(weights)
        sequence_weight = mathtools.weight(sequence)
        weight = mathtools.weight(weights)
        if self.read_talea_once_only:
            if sequence_weight < weight:
                message = 'talea is too short to read once only:'
                message += '\n{!r} in {!r}.'
                message = message.format(sequence, weights)
                raise Exception(message)
        sequence = sequencetools.Sequence(sequence).repeat_to_weight(weight)
        sequence = sequence.split(weights, cyclic=True)
        return sequence

    ### PUBLIC PROPERTIES ###

    @property
    def beam_specifier(self):
        r'''Gets beam specifier.

        ..  container:: example

            Beams each division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

        ..  container:: example

            Beams divisions together:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_divisions_together=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        \set stemLeftBeamCount = #0
                        \set stemRightBeamCount = #2
                        c'16 [
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 4/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 3/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 4/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #0
                        c'16 ]
                    }
                }

        ..  container:: example

            Beams nothing:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=False,
                ...         beam_divisions_together=False,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 4/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 3/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 4/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                }

        ..  container:: example

            Does not beam rests:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 1, 1, -1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16 ]
                        r16
                        c'16 [
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16
                        r16
                        c'16 [
                        c'16
                        c'16 ]
                        r16
                        c'16 [
                        c'16 ]
                    }
                    {
                        \time 3/8
                        c'16
                        r16
                        c'16 [
                        c'16
                        c'16 ]
                        r16
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16 ]
                        r16
                        c'16 [
                        c'16
                        c'16 ]
                        r16
                    }
                }

        ..  container:: example

            Does beam rests:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 1, 1, -1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=True,
                ...         beam_rests=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        r16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        r16
                        c'16
                        c'16
                        c'16
                        r16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 3/8
                        c'16 [
                        r16
                        c'16
                        c'16
                        c'16
                        r16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16
                        r16
                        c'16
                        c'16
                        c'16
                        r16 ]
                    }
                }

        ..  container:: example

            Beams rests with stemlets:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 1, 1, -1],
                ...         denominator=16,
                ...         ),
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=True,
                ...         beam_rests=True,
                ...         stemlet_length=0.75,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        \override Staff.Stem.stemlet-length = #0.75
                        c'16 [
                        c'16
                        c'16
                        r16
                        c'16
                        c'16 ]
                        \revert Staff.Stem.stemlet-length
                    }
                    {
                        \time 4/8
                        \override Staff.Stem.stemlet-length = #0.75
                        c'16 [
                        r16
                        c'16
                        c'16
                        c'16
                        r16
                        c'16
                        c'16 ]
                        \revert Staff.Stem.stemlet-length
                    }
                    {
                        \time 3/8
                        \override Staff.Stem.stemlet-length = #0.75
                        c'16 [
                        r16
                        c'16
                        c'16
                        c'16
                        r16 ]
                        \revert Staff.Stem.stemlet-length
                    }
                    {
                        \time 4/8
                        \override Staff.Stem.stemlet-length = #0.75
                        c'16 [
                        c'16
                        c'16
                        r16
                        c'16
                        c'16
                        c'16
                        r16 ]
                        \revert Staff.Stem.stemlet-length
                    }
                }

        Set to beam specifier or none.

        Returns beam specifier or none.
        '''
        return RhythmMaker.beam_specifier.fget(self)

    @property
    def burnish_specifier(self):
        r'''Gets burnish specifier.

        ..  container:: example

            Forces the first leaf and the last two leaves to be rests:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         right_classes=[Rest],
                ...         right_counts=[2],
                ...         outer_divisions_only=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff]) # doctest: +SKIP
                \new RhythmicStaff {
                    {
                        \time 3/8
                        r16
                        c'8 [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8 ]
                        r8.
                        r8
                    }
                }

        ..  container:: example

            Forces the first leaf of every division to be a rest:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        r16
                        c'8 [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        r4
                        c'16 [
                        c'8
                        c'16 ]
                    }
                    {
                        \time 3/8
                        r8
                        c'4
                    }
                    {
                        \time 4/8
                        r16
                        c'8 [
                        c'8.
                        c'8 ]
                    }
                }

        Set to burnish specifier or none.

        Returns burnish specifier or none.
        '''
        return self._burnish_specifier

    @property
    def division_masks(self):
        r'''Gets division masks.

        ..  container:: example

            No division masks:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8
                        c'8.
                        c'8 ]
                    }
                }

        ..  container:: example

            Silences every other output division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     division_masks=[
                ...         rhythmmakertools.SilenceMask(
                ...             pattern=patterntools.select_every([1], period=2),
                ...             ),
                ...         ],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        r2
                    }
                }

        ..  container:: example

            Sustains every other output division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     division_masks=[
                ...         rhythmmakertools.sustain_every([1], period=2),
                ...         ],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'2
                    }
                }

        ..  container:: example

            Silences every other secondary output division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1],
                ...         denominator=16,
                ...         ),
                ...     split_divisions_by_counts=[9],
                ...     division_masks=[
                ...         rhythmmakertools.SilenceMask(
                ...             pattern=patterntools.select_every([1], period=2),
                ...             ),
                ...         ],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        r8.
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 3/8
                        r4
                        c'16 [
                        c'16 ]
                    }
                    {
                        \time 4/8
                        r4..
                        c'16
                    }
                }

        ..  container:: example

            Sustains every other secondary output division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1],
                ...         denominator=16,
                ...         ),
                ...     split_divisions_by_counts=[9],
                ...     division_masks=[
                ...         rhythmmakertools.sustain_every([1], period=2),
                ...         ],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'8.
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 3/8
                        c'4
                        c'16 [
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'4..
                        c'16
                    }
                }

        Set to tuple of division masks or none.

        Returns tuple of division masks or none.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.division_masks

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier.

        Several duration spelling specifier configurations are available.

        ..  container:: example

            Spells nonassignable durations with monontonically decreasing
            durations:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         decrease_durations_monotonically=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                    {
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                    {
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                }

        ..  container:: example

            Spells nonassignable durations with monontonically increasing
            durations:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         decrease_durations_monotonically=False,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                    {
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                    {
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                }

        ..  container:: example

            Forbids no durations:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 1, 1, 1, 4, 4],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         forbidden_written_duration=None,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'16 [
                        c'16
                        c'16
                        c'16 ]
                        c'4
                        c'4
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16 ]
                        c'4
                        c'4
                    }
                }

        ..  container:: example

            Forbids durations equal to ``1/4`` or greater:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 1, 1, 1, 4, 4],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         forbidden_written_duration=Duration(1, 4),
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'8 ~
                        c'8
                        c'8 ~
                        c'8 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'8 ~
                        c'8
                        c'8 ~
                        c'8 ]
                    }
                }

            Rewrites forbidden durations with smaller durations tied together.

        ..  container:: example

            Spells all durations metrically:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 4],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'8. ~ [
                        c'8 ]
                        c'4
                        c'16 ~ [
                        c'16 ~
                        c'16 ~ ]
                    }
                    {
                        c'8
                        c'4
                        c'8. ~ [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        c'16 ~ [
                        c'16 ~
                        c'16
                        c'8. ~
                        c'8 ]
                        c'4
                    }
                }

        ..  container:: example

            Spells unassignable durations metrically:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 4],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically='unassignable',
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'8. ~ [
                        c'8 ]
                        c'4
                        c'8. ~
                    }
                    {
                        c'8
                        c'4
                        c'8. ~ [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        c'8. [
                        c'8. ~
                        c'8 ]
                        c'4
                    }
                }

        ..  container:: example

            Rewrites meter:

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 4],
                ...         denominator=16,
                ...         ),
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         rewrite_meter=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4), (3, 4)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'4 ~
                        c'16 [
                        c'8. ~
                        c'16
                        c'8. ~ ]
                    }
                    {
                        c'8 [
                        c'8 ~
                        c'8
                        c'8 ~
                        c'8.
                        c'16 ~ ]
                    }
                    {
                        c'8. [
                        c'16 ~ ]
                        c'4
                        c'4
                    }
                }

        Set to duration spelling specifier or none.

        Returns duration spelling specifier or none.
        '''
        return RhythmMaker.duration_spelling_specifier.fget(self)

    @property
    def extra_counts_per_division(self):
        r'''Gets extra counts per division.

        ..  container:: example

            No extra counts per division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8
                        c'8.
                        c'8 ]
                    }
                }

        ..  container:: example

            Adds one extra count to every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     extra_counts_per_division=[0, 1],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 8/9 {
                            c'4
                            c'16 [
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'16
                            c'4
                            c'16
                        }
                    }
                    {
                        \time 4/8
                        \times 8/9 {
                            c'8 [
                            c'8. ]
                            c'4
                        }
                    }
                }

        ..  container:: example

            Adds two extra counts to every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     extra_counts_per_division=[0, 2],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 4/5 {
                            c'4
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'4
                            c'16 [
                            c'16 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        \times 4/5 {
                            c'16 [
                            c'8. ]
                            c'4
                            c'16 [
                            c'16 ]
                        }
                    }
                }

            The duration of each added count equals the duration
            of each count in the rhythm-maker's input talea.

        ..  container:: example

            Removes one count from every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     extra_counts_per_division=[0, -1],
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 8/7 {
                            c'4
                            c'16 [
                            c'8 ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8. [
                            c'8. ~ ]
                        }
                    }
                    {
                        \time 4/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 8/7 {
                            c'16 [
                            c'16
                            c'8
                            c'8. ]
                        }
                    }
                }

        Set to integer tuple or none.

        Returns integer tuple or none.
        '''
        if self._extra_counts_per_division:
            return list(self._extra_counts_per_division)

    @property
    def helper_functions(self):
        r'''Gets helper functions.

        Set to dictionary or none.

        Returns dictionary or none.
        '''
        return self._helper_functions

    @property
    def logical_tie_masks(self):
        r'''Gets logical tie masks.

        ..  container:: example

            Silences every third logical tie:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     logical_tie_masks=[
                ...         rhythmmakertools.silence_every([2], period=3),
                ...         ],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8 ]
                        r8.
                    }
                    {
                        c'4
                        c'16
                        r16
                    }
                    {
                        r16
                        c'8. [
                        c'8 ~ ]
                    }
                    {
                        c'8
                        r16
                        c'8 [
                        c'16 ]
                    }
                }

        ..  container:: example

            Silences the first and last logical ties:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     logical_tie_masks=[
                ...         rhythmmakertools.silence_first(),
                ...         rhythmmakertools.silence_last(),
                ...         ],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        r16
                        c'8 [
                        c'8. ]
                    }
                    {
                        c'4
                        c'16 [
                        c'16 ~ ]
                    }
                    {
                        c'16 [
                        c'8.
                        c'8 ~ ]
                    }
                    {
                        c'8 [
                        c'16
                        c'8 ]
                        r16
                    }
                }

        Returns patterns or none.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.logical_tie_masks

    @property
    def read_talea_once_only(self):
        r'''Is true when rhythm-maker should read talea once only.

        ..  container:: example

            Reads talea cyclically:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        c'4
                        c'16 [
                        c'16 ~ ]
                    }
                    {
                        c'16 [
                        c'8.
                        c'8 ~ ]
                    }
                    {
                        c'8 [
                        c'16
                        c'8
                        c'16 ]
                    }
                }

        ..  container:: example

            Reads talea once only:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     read_talea_once_only=True,
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            Calling rhythm_maker on these divisions raises an exception because talea
            is too short to read once only:

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> rhythm_maker(divisions)
                Traceback (most recent call last):
                    ...
                Exception: talea is too short to read once only:
                CyclicTuple([1, 2, 3, 4]) in [6, 6, 6, 6].

        Set to true to ensure talea is long enough to cover all divisions
        without repeating.

        Provides way of using talea noncyclically when, for example,
        interpolating from short durations to long durations.

        Set to true, false or none.

        Defaults to none.

        Returns true, false or none.
        '''
        return self._read_talea_once_only

    @property
    def rest_tied_notes(self):
        r'''Is true when rhythm-maker should leave the head of each logical
        tie but change tied notes to rests and remove ties.

        ..  container:: example

            Does not rest tied notes:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        c'4
                        c'16 [
                        c'16 ~ ]
                    }
                    {
                        c'16 [
                        c'8.
                        c'8 ~ ]
                    }
                    {
                        c'8 [
                        c'16
                        c'8
                        c'16 ]
                    }
                }

        ..  container:: example

            Rests tied notes:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     rest_tied_notes=True,
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        c'4
                        c'16 [
                        c'16 ]
                    }
                    {
                        r16
                        c'8. [
                        c'8 ]
                    }
                    {
                        r8
                        c'16 [
                        c'8
                        c'16 ]
                    }
                }

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._rest_tied_notes

    @property
    def split_divisions_by_counts(self):
        r'''Gets secondary divisions.

        Secondary divisions impose a cyclic split operation on divisions.

        ..  container:: example

            Here's a talea equal to two thirty-second notes repeating
            indefinitely. Output equals four divisions of 12 thirty-second
            notes each:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[2],
                ...         denominator=32,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

        ..  container:: example

            Here's the same talea with secondary divisions set to split the
            divisions every 17 thirty-second notes. The rhythm_maker makes six
            divisions with durations equal, respectively, to 12, 5, 7, 10, 2
            and 12 thirty-second notes.

            Note that ``12 + 5 = 17`` and ``7 + 10 = 17``:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[2],
                ...         denominator=32,
                ...         ),
                ...     split_divisions_by_counts=[17],
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'32 ~ ]
                        c'32 [
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16 ]
                        c'16
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

            Additional divisions created when using `split_divisions_by_counts`
            are subject to `extra_counts_per_division` just like other
            divisions.

        ..  container:: example

            This example adds one extra thirty-second note to every other
            division. The durations of the divisions remain the same as in the
            previous example. But now every other division is tupletted:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[2],
                ...         denominator=32,
                ...         ),
                ...     split_divisions_by_counts=[17],
                ...     extra_counts_per_division=[0, 1],
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                        {
                            c'16 [
                            c'16
                            c'16
                            c'32 ~ ]
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 10/11 {
                            c'32 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                        {
                            c'16
                        }
                    }
                    {
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 12/13 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'32 ]
                        }
                    }
                }

        Set to tuple of positive integers or none.

        Returns tuple of positive integers or none.
        '''
        return self._split_divisions_by_counts

    @property
    def talea(self):
        r'''Gets talea.

        ..  container:: example

            No talea:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker()

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        c'4.
                    }
                    {
                        c'4.
                    }
                    {
                        c'4.
                    }
                }

        ..  container:: example

            Talea equal to durations ``1/16``, ``2/16``, ``3/16``, ``4/16``
            repeating indefinitely:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[1, 2, 3, 4],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        c'4
                        c'16 [
                        c'16 ~ ]
                    }
                    {
                        c'16 [
                        c'8.
                        c'8 ~ ]
                    }
                    {
                        c'8 [
                        c'16
                        c'8
                        c'16 ]
                    }
                }

        Set to talea or none.

        Returns talea or none.
        '''
        return self._talea

    @property
    def tie_specifier(self):
        r'''Gets tie specifier.

        ..  container:: example

            Does not tie across divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 3, 3, 3],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ]
                    }
                }

        ..  container:: example

            Ties across divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 3, 3, 3],
                ...         denominator=16,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ~ ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ~ ]
                    }
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ~ ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ]
                    }
                }

        ..  container:: example

            Patterns ties across divisions:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0],
                ...     period=2,
                ...     )
                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 3, 3, 3],
                ...         denominator=16,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=pattern,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ~ ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4 ~
                        c'16 [
                        c'8. ~ ]
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8. ]
                    }
                }

        ..  container:: example

            Uses Messiaen-style ties:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, 3, 3, 3],
                ...         denominator=16,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         use_messiaen_style_ties=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'4
                        c'16 \repeatTie [
                        c'8. ]
                    }
                    {
                        \time 3/8
                        c'8. \repeatTie [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4 \repeatTie
                        c'16 \repeatTie [
                        c'8. ]
                    }
                    {
                        \time 3/8
                        c'8. \repeatTie [
                        c'8. ]
                    }
                }

        ..  container:: example

            Ties consecutive notes:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[5, -3, 3, 3],
                ...         denominator=16,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_consecutive_notes=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'4 ~
                        c'16
                        r8.
                    }
                    {
                        \time 3/8
                        c'8. ~ [
                        c'8. ~ ]
                    }
                    {
                        \time 4/8
                        c'4 ~
                        c'16
                        r8.
                    }
                    {
                        \time 3/8
                        c'8. ~ [
                        c'8. ]
                    }
                }

        Set to tie specifier or none.

        Returns tie specifier.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.tie_specifier

    @property
    def tie_split_notes(self):
        r'''Is true when talea rhythm-maker should tie split notes.
        Otherwise false.

        ..  todo:: Add examples.

        Set to true or false.

        Returns true or false.
        '''
        return self._tie_split_notes

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        ..  container:: example

            Redudant tuplets with no tuplet spelling specifier:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     extra_counts_per_division=[0, 4],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[3, 3, 6, 6],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'8. [
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 2/3 {
                            c'4.
                            c'4.
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8. [
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 2/3 {
                            c'4.
                            c'4.
                        }
                    }
                }

        ..  container:: example

            Simplifies redundant tuplets:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     extra_counts_per_division=[0, 4],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[3, 3, 6, 6],
                ...         denominator=16,
                ...         ),
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         simplify_redundant_tuplets=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'8. [
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'4
                            c'4
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'8. [
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'4
                            c'4
                        }
                    }
                }

        ..  container:: example

            Rest-filled tuplets with no tuplet spelling specifier:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     extra_counts_per_division=[1, 0],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[3, 3, -6, -6],
                ...         denominator=16,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            c'8. [
                            c'8. ]
                            r16
                        }
                    }
                    {
                        \time 4/8
                        {
                            r4
                            r16
                            r8.
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            r8.
                            c'8. [
                            c'16 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8
                            r4.
                        }
                    }
                }

        ..  container:: example

            Rewrites rest-filled tuplets:

            ::

                >>> rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
                ...     extra_counts_per_division=[1, 0],
                ...     talea=rhythmmakertools.Talea(
                ...         counts=[3, 3, -6, -6],
                ...         denominator=16,
                ...         ),
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         rewrite_rest_filled_tuplets=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            c'8. [
                            c'8. ]
                            r16
                        }
                    }
                    {
                        \time 4/8
                        {
                            r2
                        }
                    }
                    {
                        \time 3/8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/7 {
                            r8.
                            c'8. [
                            c'16 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'8
                            r4.
                        }
                    }
                }

        Set to tuplet spelling specifier or none.

        Returns tuplet spelling specifier or none.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.tuplet_spelling_specifier



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/distribution/.gitignore',
        'test_score/test_score/distribution/letter-portrait/letter-portrait-parts-cello.pdf',
        'test_score/test_score/distribution/letter-portrait/letter-portrait-parts-viola.pdf',
        'test_score/test_score/distribution/letter-portrait/letter-portrait-parts-violin-i.pdf',
        'test_score/test_score/distribution/letter-portrait/letter-portrait-parts-violin-ii.pdf',
        'test_score/test_score/distribution/letter-portrait/letter-portrait-score.pdf',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success(self, open_file_mock):
        self.create_score()
        self.install_fancy_segment_maker()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = ['--render', 'letter-portrait']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        command = ['--distribute', 'letter-portrait']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        self.compare_captured_output(r'''
        Distributing 'letter-portrait'
            score.pdf --> letter-portrait-score.pdf
            parts-cello.pdf --> letter-portrait-parts-cello.pdf
            parts-viola.pdf --> letter-portrait-parts-viola.pdf
            parts-violin-i.pdf --> letter-portrait-parts-violin-i.pdf
            parts-violin-ii.pdf --> letter-portrait-parts-violin-ii.pdf
        ''')
        self.compare_path_contents(
            self.distribution_path,
            self.expected_files,
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_run_01():

    foo = "c'8 d'8 r8 r8 <e' g'>8 <f' a'>8 g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8"
    staff = Staff(foo)
    groups = iterate(staff[:]).by_run((Note, Chord))
    groups = list(groups)

    assert groups[0] == (staff[0], staff[1])
    assert groups[1] == (staff[4], staff[5], staff[6], staff[7])
    assert groups[2] == (staff[10], staff[11])



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Parentage__get_governor_01( ):
    r'''Returns the last sequential container in the parentage of client
    such that the next element in the parentage of client is
    either a simultaneous container or None.
    '''

    voice = Voice([Container([Voice("c'8 d'8"), Voice("e'8 f'8")])])
    voice[0].is_simultaneous = True
    voice[0][0].name = 'voice 1'
    voice[0][1].name = 'voice 2'

    r'''
    \new Voice {
        <<
            \context Voice = "voice 1" {
                c'8
                d'8
            }
            \context Voice = "voice 2" {
                e'8
                f'8
            }
        >>
    }
    '''

    leaves = select(voice).by_leaf()
    assert inspect_(leaves[0]).get_parentage()._get_governor() is voice[0][0]
    assert inspect_(leaves[1]).get_parentage()._get_governor() is voice[0][0]
    assert inspect_(leaves[2]).get_parentage()._get_governor() is voice[0][1]
    assert inspect_(leaves[3]).get_parentage()._get_governor() is voice[0][1]


def test_selectiontools_Parentage__get_governor_02( ):
    r'''Unicorporated leaves have no governor.
    '''

    note = Note(0, (1, 8))
    assert inspect_(note).get_parentage()._get_governor() is None


def test_selectiontools_Parentage__get_governor_03( ):
    r'''Returns the last sequential container in the parentage of client
    such that the next element in the parentage of client is
    either a simultaneous container or None.
    '''

    staff = Staff([Voice([Container("c'8 d'8 e'8 f'8")])])

    r'''
    \new Staff {
        \new Voice {
            {
                c'8
                d'8
                e'8
                f'8
            }
        }
    }
    '''

    leaves = select(staff).by_leaf()
    assert inspect_(leaves[0]).get_parentage()._get_governor() is staff
    assert inspect_(leaves[1]).get_parentage()._get_governor() is staff
    assert inspect_(leaves[2]).get_parentage()._get_governor() is staff
    assert inspect_(leaves[3]).get_parentage()._get_governor() is staff


def test_selectiontools_Parentage__get_governor_04( ):
    r'''Returns the last sequential container in the parentage of client
    such that the next element in the parentage of client is
    either a simultaneous container or None.
    '''

    staff = Staff([Voice([Container("c'8 d'8 e'8 f'8")])])

    r'''
    \new Staff {
        \new Voice {
            {
                c'8
                d'8
                e'8
                f'8
            }
        }
    }
    '''

    assert inspect_(staff[0][0]).get_parentage()._get_governor() is staff
    assert inspect_(staff[0]).get_parentage()._get_governor() is staff
    assert inspect_(staff).get_parentage()._get_governor() is staff



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental_is_abbreviation_01():

    assert pitchtools.Accidental.is_abbreviation('')
    assert pitchtools.Accidental.is_abbreviation('s')
    assert pitchtools.Accidental.is_abbreviation('ss')
    assert pitchtools.Accidental.is_abbreviation('f')
    assert pitchtools.Accidental.is_abbreviation('ff')
    assert pitchtools.Accidental.is_abbreviation('qs')
    assert pitchtools.Accidental.is_abbreviation('tqs')
    assert pitchtools.Accidental.is_abbreviation('qf')
    assert pitchtools.Accidental.is_abbreviation('tqf')


def test_pitchtools_Accidental_is_abbreviation_02():

    assert not pitchtools.Accidental.is_abbreviation(8)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Annotation___call___01():

    staff = Staff("c'8 d'8 e'8 f'8")
    annotation = indicatortools.Annotation('foo')
    attach(annotation, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_Scale___init___01():
    r'''Initialize with tonic and mode strings.
    '''

    scale = tonalanalysistools.Scale('g', 'major')
    assert scale.key_signature == KeySignature('g', 'major')


def test_tonalanalysistools_Scale___init___02():
    r'''Initialize with key signature instance.
    '''

    key_signature = KeySignature('g', 'major')
    scale = tonalanalysistools.Scale(key_signature)
    assert scale.key_signature == KeySignature('g', 'major')


def test_tonalanalysistools_Scale___init___03():
    r'''Initialize with other scale instance.
    '''

    scale = tonalanalysistools.Scale('g', 'major')
    new = tonalanalysistools.Scale(scale)
    assert new.key_signature == KeySignature('g', 'major')



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_01():

    string = 'Duation._group_nonreduced_fractions_by_implied_prolation([])'
    assert pytest.raises(Exception, string)


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_02():

    fractions = [(1, 4)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [[mathtools.NonreducedFraction(1, 4)]]


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_03():

    fractions = [(1, 4), (1, 4), (1, 8)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [[
        mathtools.NonreducedFraction(1, 4),
        mathtools.NonreducedFraction(1, 4),
        mathtools.NonreducedFraction(1, 8),
        ]]


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_04():

    fractions = [(1, 4), (1, 3), (1, 8)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [
        [mathtools.NonreducedFraction(1, 4)],
        [mathtools.NonreducedFraction(1, 3)],
        [mathtools.NonreducedFraction(1, 8)],
        ]


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_05():

    fractions = [(1, 4), (1, 2), (1, 3)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [
        [mathtools.NonreducedFraction(1, 4),
            mathtools.NonreducedFraction(1, 2)],
        [mathtools.NonreducedFraction(1, 3)],
        ]


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_06():

    fractions = [(1, 4), (1, 2), (1, 3), (1, 6), (1, 5)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [
        [mathtools.NonreducedFraction(1, 4),
            mathtools.NonreducedFraction(1, 2)],
        [mathtools.NonreducedFraction(1, 3),
            mathtools.NonreducedFraction(1, 6)],
        [mathtools.NonreducedFraction(1, 5)],
        ]


def test_durationtools_Duration__group_nonreduced_fractions_by_implied_prolation_07():

    fractions = [(1, 24), (2, 24), (3, 24), (4, 24), (5, 24), (6, 24)]
    duration = Duration._group_nonreduced_fractions_by_implied_prolation(fractions)
    assert duration == [[
        mathtools.NonreducedFraction(1, 24),
        mathtools.NonreducedFraction(2, 24),
        mathtools.NonreducedFraction(3, 24),
        mathtools.NonreducedFraction(4, 24),
        mathtools.NonreducedFraction(5, 24),
        mathtools.NonreducedFraction(6, 24),
        ]]



# -*- coding: utf-8 -*-
from abjad.tools import durationtools


def make_percussion_note(pitch, total_duration, max_note_duration=(1, 8)):
    r'''Makes short note with `max_note_duration` followed
    by rests together totaling `total_duration`.

    ..  container:: example

            >>> leaves = scoretools.make_percussion_note(2, (1, 4), (1, 8))
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                d'8
                r8
            }

    ..  container:: example

            >>> leaves = scoretools.make_percussion_note(2, (1, 64), (1, 8))
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                d'64
            }

    ..  container:: example

            >>> leaves = scoretools.make_percussion_note(2, (5, 64), (1, 8))
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                d'16
                r64
            }

    ..  container:: example

            >>> leaves = scoretools.make_percussion_note(2, (5, 4), (1, 8))
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                d'8
                r1
                r8
            }

    Returns list of newly constructed note followed by zero or
    more newly constructed rests.

    Durations of note and rests returned will sum to `total_duration`.

    Duration of note returned will be no greater than `max_note_duration`.

    Duration of rests returned will sum to note duration taken
    from `total_duration`.

    Useful for percussion music where attack duration is negligible
    and tied notes undesirable.
    '''
    from abjad.tools import scoretools
    from abjad.tools import scoretools
    from abjad.tools import selectiontools

    # check input
    total_duration = durationtools.Duration(total_duration)
    max_note_duration = durationtools.Duration(max_note_duration)

    # make note and rest
    if max_note_duration < total_duration:
        rest_duration = total_duration - max_note_duration
        rests = scoretools.make_tied_leaf(
            scoretools.Rest,
            rest_duration,
            pitches=None,
            )
        notes = scoretools.make_tied_leaf(
            scoretools.Note,
            max_note_duration,
            pitches=pitch,
            )
    else:
        notes = scoretools.make_tied_leaf(
            scoretools.Note,
            total_duration,
            pitches=pitch,
            tie_parts=False,
            )
        if 1 < len(notes):
            new_notes = []
            new_notes.append(notes[0])
            for i in range(1, len(notes)):
                rest = scoretools.Rest(notes[i])
                new_notes.append(rest)
            notes = new_notes
        rests = []

    # return list of percussion note followed by rest
    result = notes + rests
    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
import functools
import numbers
from abjad.tools import mathtools
from abjad.tools.pitchtools.PitchClass import PitchClass


@functools.total_ordering
class NumberedPitchClass(PitchClass):
    '''Numbered pitch-class.

    ..  container:: example

        Initializes from number of semitones:

        ::

            >>> NumberedPitchClass(13)
            NumberedPitchClass(1)

    ..  container:: example

        Initializes from pitch name.

        ::

            >>> NumberedPitchClass('d')
            NumberedPitchClass(2)

    ..  container:: example

        Initializes from named pitch.

        ::

            >>> NumberedPitchClass(NamedPitch('g,'))
            NumberedPitchClass(7)

    ..  container:: example

        Initializes from numbered pitch.

        ::

            >>> NumberedPitchClass(NumberedPitch(15))
            NumberedPitchClass(3)

    ..  container:: example

        Initializes from named pitch-class.

        ::

            >>> NumberedPitchClass(NamedPitchClass('e'))
            NumberedPitchClass(4)

    ..  container:: example

        Initializes from numbered pitch-class.

        ::

            >>> NumberedPitchClass('C#5')
            NumberedPitchClass(1)

    ..  container:: example

        Initializes from numbered pitch-class.

        ::

            >>> NumberedPitchClass(Note("a'8."))
            NumberedPitchClass(9)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, argument=None):
        from abjad.tools import pitchtools
        prototype = (numbers.Number, pitchtools.NumberedPitch, type(self))
        if isinstance(argument, prototype):
            self._initialize_by_number(float(argument))
        elif isinstance(argument, pitchtools.NamedPitch):
            self._initialize_by_named_pitch(argument)
        elif isinstance(argument, pitchtools.NamedPitchClass):
            self._initialize_by_named_pitch_class(argument)
        elif isinstance(argument, str):
            self._initialize_by_string(argument)
        elif pitchtools.Pitch.is_pitch_carrier(argument):
            self._initialize_by_pitch_carrier(argument)
        elif argument is None:
            self._initialize_by_number(0)
        else:
            message = 'can not instantiate {} from {!r}.'
            message = message.format(type(self).__name__, argument)
            raise TypeError(message)

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds `argument` to numbered pitch-class.

        ..  container:: example

            ::

                >>> pitch_class = NumberedPitchClass(9)
                >>> interval = NumberedInterval(4)
                >>> pitch_class + interval
                NumberedPitchClass(1)

        Returns new numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NumberedInterval(argument)
        return type(self)(self.number + interval.number % 12)

    def __copy__(self, *arguments):
        r'''Copies numbered pitch-class.

        ..  container:: example

            ::

                >>> import copy
                >>> pitch_class = NumberedPitchClass(9)
                >>> copy.copy(pitch_class)
                NumberedPitchClass(9)

        Returns new numbered pitch-class.
        '''
        return type(self)(self)

    def __eq__(self, argument):
        r'''Is true when `argument` is a numbered pitch-class with pitch-class
        number equal to that of this numbered pitch-class.

        ..  container:: example

            ::

                >>> pitch_class_1 = NumberedPitchClass(9)
                >>> pitch_class_2 = NumberedPitchClass(3)
                >>> pitch_class_1 == pitch_class_1
                True

        Otherwise false:

        ..  container:: example

            ::

                >>> pitch_class_1 == pitch_class_2
                False

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.number == argument.number
        return self.number == argument

    def __float__(self):
        r'''Changes numbered pitch-class to float.

        ..  container:: example

            ::

                >>> pitch_class = NumberedPitchClass(9)
                >>> float(pitch_class)
                9.0

        Returns float.
        '''
        return float(self.number)

    def __format__(self, format_specification=''):
        r'''Formats numbered pitch-class.

        ..  container:: example

            ::

                >>> format(NumberedPitchClass(13))
                'pitchtools.NumberedPitchClass(1)'

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        superclass = super(NumberedPitchClass, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes numbered pitch-class.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NumberedPitchClass, self).__hash__()

    def __int__(self):
        r'''Changes numbered pitch-class to integer.

        ..  container:: example

            ::

                >>> pitch_class = NumberedPitchClass(9)
                >>> int(pitch_class)
                9

        Returns integer.
        '''
        return self.number

    def __lt__(self, argument):
        r'''Is true when `argument` is a numbered pitch-class with a pitch
        number greater than that of this numberd pitch-class.

        ..  container:: example

            Compares less than:

            ::

                >>> NumberedPitchClass(1) < NumberedPitchClass(2)
                True

        ..  container:: example

            Does not compare less than:

            ::

                >>> NumberedPitchClass(2) < NumberedPitchClass(1)
                False

        Raises type error when `argument` is not a numbered pitch-class.
        '''
        if not isinstance(argument, type(self)):
            message = 'can not compare numbered pitch-class to {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return self.number < argument.number

    def __neg__(self):
        r'''Negates numbered pitch-class.

        ..  container:: example

            ::

                >>> pitch_class = NumberedPitchClass(9)
                >>> -pitch_class
                NumberedPitchClass(3)

        Returns new numbered pitch-class.
        '''
        #return type(self)(-self.pitch_class_number)
        return type(self)(-self.number)

    def __str__(self):
        r'''Gets string representation of numbered pitch-class.

        Returns string.
        '''
        #return str(self.pitch_class_number)
        return str(self.number)

    def __sub__(self, argument):
        r'''Subtracts `argument` from numbered pitch-class.

        Subtraction defined against both numbered intervals
        and against other pitch-classes.

        Returns numbered inversion-equivalent interval-class.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
#            interval_class_number = abs(
#                self.pitch_class_number -
#                argument.pitch_class_number
#                )
            interval_class_number = abs(
                self.number - argument.number
                )
            if 6 < interval_class_number:
                interval_class_number = 12 - interval_class_number
            return pitchtools.NumberedInversionEquivalentIntervalClass(
                interval_class_number)
        interval_class = pitchtools.NumberedInversionEquivalentIntervalClass(
            argument)
        #return type(self)(self.pitch_class_number - interval_class.number % 12)
        return type(self)(self.number - interval_class.number % 12)

    ### PRIVATE METHODS ###

    def _initialize_by_named_pitch(self, argument):
        self._number = argument.pitch_class_number

    def _initialize_by_named_pitch_class(self, argument):
        self._number = argument.pitch_class_number

    def _initialize_by_number(self, argument):
        argument = round((float(argument) % 12) * 4) / 4
        div, mod = divmod(argument, 1)
        if mod == 0.75:
            div += 1
        elif mod == 0.5:
            div += 0.5
        div %= 12
        self._number = mathtools.integer_equivalent_number_to_integer(div)

    def _initialize_by_pitch_carrier(self, argument):
        from abjad.tools import pitchtools
        named_pitch = pitchtools.NamedPitch.from_pitch_carrier(argument)
        self._initialize_by_named_pitch(named_pitch)

    def _initialize_by_string(self, argument):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(argument)
        self._initialize_by_named_pitch_class(named_pitch_class)

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Gets accidental.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).accidental
                Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self.alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Gets alteration in semitones.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).alteration_in_semitones
                1

            ::

                >>> NumberedPitchClass(10.5).alteration_in_semitones
                -0.5

        Returns integer or float.
        '''
        return (self.number -
            self._diatonic_pitch_class_number_to_pitch_class_number[
                self.diatonic_pitch_class_number])

    @property
    def diatonic_pitch_class_name(self):
        r'''Gets diatonic pitch-class name.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).diatonic_pitch_class_name
                'c'

        Returns string.
        '''
        return self.pitch_class_name[0]

    @property
    def diatonic_pitch_class_number(self):
        r'''Gets diatonic pitch-class number.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).diatonic_pitch_class_number
                0

        Returns integer.
        '''
        return self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
            self.diatonic_pitch_class_name]

    @property
    def named_pitch_class(self):
        r'''Gets named pitch-class.

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).named_pitch_class
                NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def number(self):
        r'''Gets number.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).number
                1

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).number
                1

        '''
        return self._number

    @property
    def numbered_pitch_class(self):
        r'''Gets numbered pitch-class.

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).numbered_pitch_class
                NumberedPitchClass(1)

        Returns new numbered pitch-class.
        '''
        return type(self)(self)

    @property
    def pitch_class_label(self):
        r'''Gets pitch-class / octave label.

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).pitch_class_label
                'C#'

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            )

    @property
    def pitch_class_name(self):
        r'''Gets pitch-class name.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).pitch_class_name
                'cs'

        Returns string.
        '''
        from abjad import abjad_configuration
        accidental_spelling = abjad_configuration['accidental_spelling']
        if accidental_spelling == 'mixed':
            return self._pitch_class_number_to_pitch_class_name[self.number]
        elif accidental_spelling == 'sharps':
            return self._pitch_class_number_to_pitch_class_name_with_sharps[
                self.number]
        elif accidental_spelling == 'flats':
            return self._pitch_class_number_to_pitch_class_name_with_flats[
                self.number]
        else:
            message = 'unknown accidental spelling: {!r}.'
            message = message.format(accidental_spelling)
            raise ValueError(message)

    @property
    def pitch_class_number(self):
        r'''Gets pitch-class number.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).pitch_class_number
                1

        Returns number.
        '''
        return self._number

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        '''Applies `accidental` to numbered pitch-class.

        ..  container:: example

            ::

                >>> NumberedPitchClass(1).apply_accidental('flat')
                NumberedPitchClass(0)

        Returns new numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        semitones = self.number + accidental.semitones
        return type(self)(semitones)

    def invert(self, axis=None):
        r'''Inverts numbered pitch-class.

        Interprets axis of inversion equal to pitch-class 0.

        Returns new numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        axis = axis or pitchtools.NumberedPitch('c')
        axis = pitchtools.NumberedPitch(axis)
        this = pitchtools.NumberedPitch(self)
        interval = this - axis
        result = axis.transpose(interval)
        result = type(self)(result)
        return result

    def multiply(self, n=1):
        r'''Multiplies pitch-class number by `n`.

        ..  container:: example

            ::

                >>> NumberedPitchClass(11).multiply(3)
                NumberedPitchClass(9)

        Returns new numbered pitch-class.
        '''
        return type(self)(n * self.number)

    def transpose(self, n=0):
        r'''Transposes numbered pitch-class by index `n`.

        Returns new numbered pitch-class.
        '''
        return type(self)(self.number + n)



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeNode_improper_parentage_01():

    container = rhythmtreetools.RhythmTreeContainer()
    assert container.improper_parentage == (container,)

    leaf = rhythmtreetools.RhythmTreeLeaf()
    assert leaf.improper_parentage == (leaf,)

    container.append(leaf)
    assert leaf.improper_parentage == (leaf, container)

    subcontainer = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf])
    container.append(subcontainer)
    assert leaf.improper_parentage == (leaf, subcontainer, container)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import durationtools


def test_schemetools_SchemeMoment_duration_01():

    scheme_moment = schemetools.SchemeMoment((1, 68))

    assert scheme_moment.duration == durationtools.Duration((1, 68))



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_WeightedSearchTree___call___01():
    definition = {
        'divisors': (2, 3, 5, 7),
        'max_depth': 3,
        'max_divisions': 2,
        }
    search_tree = quantizationtools.WeightedSearchTree(definition)
    q_grid = quantizationtools.QGrid()
    a = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(0,      ['A'], index=1), 0, 1)
    b = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1)
    c = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1)
    d = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1)
    e = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1)
    f = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1)
    g = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1)
    h = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1)
    i = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1)
    j = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1)
    k = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1,      ['K'], index=11), 0, 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g, h, i, j, k])
    q_grids = search_tree(q_grid)

    assert [q_grid.root_node.rtm_format for q_grid in q_grids] == [
        '(1 (1 1))',
        '(1 (2 1))',
        '(1 (1 2))',
        '(1 (4 1))',
        '(1 (3 2))',
        '(1 (2 3))',
        '(1 (1 4))',
        '(1 (6 1))',
        '(1 (5 2))',
        '(1 (4 3))',
        '(1 (3 4))',
        '(1 (2 5))',
        '(1 (1 6))'
    ]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_LilyPondCommand___setattr___01():
    r'''Slots constrain LilyPond commands.
    '''

    lilypond_command = indicatortools.LilyPondCommand('break')

    assert pytest.raises(AttributeError, "lilypond_command.foo = 'bar'")



# -*- coding: utf-8 -*-
from __future__ import print_function
import copy
import os
import subprocess
import sys
from abjad.tools import documentationtools
from abjad.tools import systemtools
from abjad.tools import lilypondfiletools
from abjad.tools.abjadbooktools.ImageOutputProxy import ImageOutputProxy


class LilyPondOutputProxy(ImageOutputProxy):
    r'''A LilyPond output proxy.

    ::

        >>> from abjad.tools import abjadbooktools
        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> proxy = abjadbooktools.LilyPondOutputProxy(staff)
        >>> print(format(proxy))
        abjadbooktools.LilyPondOutputProxy(
            lilypondfiletools.LilyPondFile(
                comments=[],
                global_staff_size=12,
                includes=[],
                items=[
                    lilypondfiletools.Block(
                        name='header',
                        ),
                    lilypondfiletools.Block(
                        name='layout',
                        ),
                    lilypondfiletools.Block(
                        name='score',
                        ),
                    ],
                lilypond_language_token=lilypondfiletools.LilyPondLanguageToken(),
                lilypond_version_token=lilypondfiletools.LilyPondVersionToken(
                    version_string='2.19.0',
                    ),
                )
            )

    ::

        >>> proxy.as_latex(relative_output_directory='assets')
        ['\\noindent\\includegraphics{assets/lilypond-0b731cedacea34e85fbb92b66b42b40b.pdf}']

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Output Proxies'

    __slots__ = (
        '_pages',
        '_payload',
        '_stylesheet',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        payload,
        image_layout_specifier=None,
        image_render_specifier=None,
        ):
        from abjad.tools import abjadbooktools
        ImageOutputProxy.__init__(
            self,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            )
        payload = copy.deepcopy(payload)
        if image_render_specifier is None:
            image_render_specifier = abjadbooktools.ImageRenderSpecifier()
        if (
            not image_render_specifier.stylesheet and
            not image_render_specifier.no_stylesheet
            ):
            payload = documentationtools.make_reference_manual_lilypond_file(
                payload)
        lilypond_file = payload
        assert isinstance(lilypond_file, lilypondfiletools.LilyPondFile)
        if (lilypond_file.layout_block and
            not len(lilypond_file.layout_block.items)):
            lilypond_file.items.remove(lilypond_file.layout_block)
        if (lilypond_file.paper_block and
            not len(lilypond_file.paper_block.items)):
            lilypond_file.items.remove(lilypond_file.paper_block)
        if lilypond_file.header_block is None:
            header_block = lilypondfiletools.Block(name='header')
            lilypond_file.items.insert(0, header_block)
        lilypond_file.header_block.tagline = False
        lilypond_file._date_time_token = None
        token = lilypondfiletools.LilyPondVersionToken("2.19.0")
        lilypond_file._lilypond_version_token = token
        if (
            image_render_specifier.stylesheet and
            not image_render_specifier.no_stylesheet
            ):
            if not lilypond_file.includes:
                lilypond_file._use_relative_includes = True
                includes = [image_render_specifier.stylesheet]
                lilypond_file._includes = tuple(includes)
        self._payload = lilypond_file

    ### PRIVATE METHODS ###

    def _render_pdf_source(
        self,
        temporary_directory_path,
        ):
        ly_file_path = os.path.join(
            temporary_directory_path,
            self.file_name_without_extension + '.ly',
            )
        source = format(self.payload)
        with open(ly_file_path, 'w') as file_pointer:
            file_pointer.write(source)
        systemtools.IOManager.run_lilypond(ly_file_path)
        pdf_file_path = os.path.join(
            temporary_directory_path,
            self.file_name_without_extension + '.pdf',
            )
        if not os.path.exists(pdf_file_path):
            print(format(self.payload))
            raise AssertionError
        assert systemtools.IOManager.find_executable('pdfcrop')
        command = 'pdfcrop {path} {path}'.format(path=pdf_file_path)

#        exit_code = subprocess.call(
#            command,
#            shell=True,
#            stdout=subprocess.PIPE,
#            stderr=subprocess.PIPE,
#            )
#        assert exit_code == 0
        process = subprocess.Popen(
            command,
            shell=True,
            stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE,
            )
        stdout, stderr = process.communicate()
        if not process.returncode == 0:
            raise Exception(stdout)

        return pdf_file_path

    ### PUBLIC METHODS ###

    def as_docutils(
        self,
        configuration=None,
        output_directory=None,
        ):
        r'''Creates a docutils node representation of the output proxy.

        ::

            >>> for node in proxy.as_docutils():
            ...     print(node.pformat())
            ...
            <abjad_output_block image_layout_specifier... image_render_specifier... renderer="lilypond" xml:space="preserve">
                \version "2.19.0"
                \language "english"
            <BLANKLINE>
                #(set-global-staff-size 12)
            <BLANKLINE>
                \header {
                    tagline = ##f
                }
            <BLANKLINE>
                \layout {
                    indent = #0
                    ragged-right = ##t
                    \context {
                        \Score
                        \remove Bar_number_engraver
                        \override SpacingSpanner.strict-grace-spacing = ##t
                        \override SpacingSpanner.strict-note-spacing = ##t
                        \override SpacingSpanner.uniform-stretching = ##t
                        \override TupletBracket.bracket-visibility = ##t
                        \override TupletBracket.minimum-length = #3
                        \override TupletBracket.padding = #2
                        \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                        proportionalNotationDuration = #(ly:make-moment 1 24)
                        tupletFullLength = ##t
                    }
                }
            <BLANKLINE>
                \score {
                    \new Staff {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                }
            <BLANKLINE>

        Returns list of docutils nodes.
        '''
        from abjad.tools import abjadbooktools
        result = []
        try:
            code = format(self.payload)
            if sys.version_info[0] == 2:
                code = code.decode('utf-8')
            node = abjadbooktools.abjad_output_block(code, code)
            node['image_layout_specifier'] = self.image_layout_specifier
            node['image_render_specifier'] = self.image_render_specifier
            node['renderer'] = 'lilypond'
            result.append(node)
        except UnicodeDecodeError:
            print()
            print(type(self))
            for line in code.splitlines():
                print(repr(line))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def file_name_prefix(self):
        r'''Gets file name prefix of LilyPond output proxy.

        Returns string.
        '''
        return 'lilypond'



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__indicators__Tempo_01():

    target = Score([Staff([Note(0, 1)])])
    tempo = Tempo(textual_indication="As fast as possible")
    attach(tempo, target[0], scope=Staff)

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo "As fast as possible"
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    tempos = inspect_(leaf).get_indicators(Tempo)
    assert len(tempos) == 1


def test_lilypondparsertools_LilyPondParser__indicators__Tempo_02():

    target = Score([Staff([Note(0, 1)])])
    leaves = select(target).by_leaf()
    tempo = Tempo((1, 4), 60)
    attach(tempo, leaves[0], scope=Staff)

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo 4=60
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    tempos = inspect_(leaf).get_indicators(Tempo)
    assert len(tempos) == 1


def test_lilypondparsertools_LilyPondParser__indicators__Tempo_03():

    target = Score([Staff([Note(0, 1)])])
    leaves = select(target).by_leaf()
    tempo = Tempo((1, 4), (59, 63))
    attach(tempo, leaves[0], scope=Staff)

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo 4=59-63
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    tempos = inspect_(leaf).get_indicators(Tempo)
    assert len(tempos) == 1


def test_lilypondparsertools_LilyPondParser__indicators__Tempo_04():

    target = Score([Staff([Note(0, 1)])])
    tempo = Tempo(
        reference_duration=(1, 4),
        units_per_minute=60,
        textual_indication="Like a majestic swan, alive with youth and vigour!",
        )
    leaves = select(target).by_leaf()
    attach(tempo, leaves[0], scope=Staff)

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo "Like a majestic swan, alive with youth and vigour!" 4=60
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    tempos = inspect_(leaf).get_indicators(Tempo)
    assert len(tempos) == 1


def test_lilypondparsertools_LilyPondParser__indicators__Tempo_05():

    target = Score([Staff([Note(0, 1)])])
    tempo = Tempo(
        reference_duration=(1, 16), 
        units_per_minute=(34, 55),
        textual_indication="Faster than a thousand suns",
        )
    leaves = select(target).by_leaf()
    attach(tempo, leaves[0], scope=Staff)

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo "Faster than a thousand suns" 16=34-55
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    tempos = inspect_(leaf).get_indicators(Tempo)
    assert len(tempos) == 1



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___rmul___01():

    major_second = pitchtools.NamedInterval('major', 2)

    assert 0 * major_second == pitchtools.NamedInterval('perfect', 1)
    assert 1 * major_second == pitchtools.NamedInterval('major', 2)
    assert 2 * major_second == pitchtools.NamedInterval('major', 3)
    assert 3 * major_second == pitchtools.NamedInterval('augmented', 4)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadList_append_01():
    r'''Append tweaked note-head to chord.
    '''

    chord = Chord("<c' d'>4")
    note_head = scoretools.NoteHead("b'")
    note_head.tweak.style = 'harmonic'
    chord.note_heads.append(note_head)

    assert format(chord) == stringtools.normalize(
        r'''
        <
            c'
            d'
            \tweak style #'harmonic
            b'
        >4
        '''
        )

    assert note_head._client is chord



# -*- coding: utf-8 -*-
#! /usr/bin/env python

if __name__ == '__main__':
    from abjad import show
    from abjad.demos import mozart
    lilypond_file = mozart.make_mozart_lilypond_file()
    show(lilypond_file)



# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools


def test_rhythmtreetools_RhythmTreeLeaf___eq___01():

    a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1, is_pitched=True)
    b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1, is_pitched=True)

    assert format(a) == format(b)
    assert a != b


def test_rhythmtreetools_RhythmTreeLeaf___eq___02():

    a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1, is_pitched=True)
    b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1, is_pitched=False)
    c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2, is_pitched=True)
    d = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2, is_pitched=False)

    assert a != b
    assert a != c
    assert a != d
    assert b != c
    assert b != d
    assert c != d



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_IntervalClassSegment_is_tertian_01():

    dicseg = pitchtools.IntervalClassSegment([
        pitchtools.NamedInversionEquivalentIntervalClass('major', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('diminished', 3)])

    assert dicseg.is_tertian


def test_pitchtools_IntervalClassSegment_is_tertian_02():

    dicseg = pitchtools.IntervalClassSegment([
        pitchtools.NamedInversionEquivalentIntervalClass('major', 2),
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('diminished', 3)])

    assert not dicseg.is_tertian



# -*- coding: utf-8 -*-


def attach(
    indicator,
    component_expression,
    scope=None,
    is_annotation=None,
    name=None,
    synthetic_offset=None,
    ):
    r'''Attaches `indicator` to `component_expression`.

    ..  container:: example

        Attaches clef to first note in staff:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Clef('alto'), staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \clef "alto"
                c'4
                d'4
                e'4
                f'4
            }

    ..  container:: example

        Attaches accent to last two notes in staff:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Articulation('>'), staff[-2])
            >>> attach(Articulation('>'), staff[-1])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                d'4
                e'4 -\accent
                f'4 -\accent
            }

    Derives scope from the default scope of `indicator` when `scope` is none.

    Treats indicator as annotation when `is_annotation` is true.

    Returns none.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools
    from abjad.tools import spannertools
    from abjad.tools.topleveltools import iterate

    if hasattr(indicator, '_attachment_test_all'):
        if not indicator._attachment_test_all(component_expression):
            message = '{!r} attachment test fails for {!r}.'
            message = message.format(indicator, component_expression)
            raise Exception(message)

    if hasattr(indicator, '_attach'):
        prototype = (spannertools.Spanner, scoretools.GraceContainer)
        assert isinstance(indicator, prototype), repr(indicator)
        assert scope is None
        if isinstance(indicator, spannertools.Spanner):
            name = name or indicator.name
            indicator._name = name
            leaves = []
            try:
                for x in component_expression:
                    if isinstance(x, scoretools.Leaf):
                        leaves.append(x)
                    else:
                        leaves.extend(iterate(x).by_leaf())
            except TypeError:
                leaves.append(component_expression)
            indicator._attach(leaves)
        else:
            indicator._attach(component_expression)
        return

    component = component_expression
    prototype = (scoretools.Component, spannertools.Spanner)
    if not isinstance(component, prototype):
        message = 'must be component or spanner: {!r}.'
        message = message.format(component)
        raise Exception(message)

    if isinstance(indicator, indicatortools.IndicatorExpression):
        is_annotation = is_annotation or indicator.is_annotation
        name = name or indicator.name
        scope = scope or indicator.scope
        synthetic_offset = synthetic_offset or indicator.synthetic_offset
        indicator._detach()
        indicator = indicator.indicator

    if hasattr(indicator, '_default_scope'):
        scope = scope or indicator._default_scope

    expression = indicatortools.IndicatorExpression(
        component=component,
        indicator=indicator,
        is_annotation=is_annotation,
        name=name,
        scope=scope,
        synthetic_offset=synthetic_offset,
        )
    expression._bind_to_component(component)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__indicators__Markup_01():

    target = Staff([Note(0, 1)])
    markup = markuptools.Markup('hello!', Up)
    attach(markup, target[0])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'1 ^ \markup { hello! }
        }
        '''
        )

    string = r'''\new Staff { c'1 ^ "hello!" }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target, 'lilypond') == format(result, 'lilypond') and \
        target is not result
    assert 1 == len(inspect_(result[0]).get_markup())


def test_lilypondparsertools_LilyPondParser__indicators__Markup_02():

    target = Staff([Note(0, (1, 4))])
    markup = markuptools.Markup(['X', 'Y', 'Z', 'a b c'], Down)
    attach(markup, target[0])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
                _ \markup {
                    X
                    Y
                    Z
                    "a b c"
                    }
        }
        '''
        )

    string = r'''\new Staff { c' _ \markup { X Y Z "a b c" } }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target, 'lilypond') == format(result, 'lilypond') and \
        target is not result
    assert 1 == len(inspect_(result[0]).get_markup())


def test_lilypondparsertools_LilyPondParser__indicators__Markup_03():
    r'''Articulations following markup block are (re)lexed correctly after
    returning to the "notes" lexical state after popping the "markup lexical state.
    '''

    target = Staff([Note(0, (1, 4)), Note(2, (1, 4))])
    markup = markuptools.Markup('hello', Up)
    attach(markup, target[0])
    articulation = Articulation('.')
    attach(articulation, target[0])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 -\staccato ^ \markup { hello }
            d'4
        }
        '''
        )

    string = r'''\new Staff { c' ^ \markup { hello } -. d' }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target, 'lilypond') == format(result, 'lilypond') and \
        target is not result
    assert 1 == len(inspect_(result[0]).get_markup())


def test_lilypondparsertools_LilyPondParser__indicators__Markup_04():

    command1 = markuptools.MarkupCommand('bold', ['A', 'B', 'C'])
    command2 = markuptools.MarkupCommand('italic', '123')
    markup = markuptools.Markup((command1, command2))

    parser = LilyPondParser()
    result = parser(format(markup, 'lilypond'))

    assert isinstance(result, markuptools.Markup)
    assert format(result, 'lilypond') == format(markup, 'lilypond')


def test_lilypondparsertools_LilyPondParser__indicators__Markup_05():

    command = r'\markup { \char ##x03EE }'
    parser = LilyPondParser()
    result = parser(command)
    assert format(result, 'lilypond') == stringtools.normalize(
        r'''
        \markup {
            \char
                #1006
            }
        ''',
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_grace_containers_01():

    staff = Staff("c'8 d'8 e'8 f'8")

    grace_container = scoretools.GraceContainer(
        [Note("cs'16")], kind='grace')
    attach(grace_container, staff[1])

    after_grace_container = scoretools.GraceContainer(
        [Note("ds'16")], kind='after')
    attach(after_grace_container, staff[1])

    grace_containers = inspect_(staff[1]).get_grace_containers()

    assert len(grace_containers) == 2

    assert grace_container in grace_containers
    assert after_grace_container in grace_containers



# -*- coding: utf-8 -*-


def offset_happens_before_timespan_starts(
    timespan=None,
    offset=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `offset` happens
    before `timespan` starts.

    ..  container:: example

        **Example 1.** Makes time relation indicating that `offset`
        happens before `timespan` starts:

        ::

            >>> relation = timespantools.offset_happens_before_timespan_starts()
            >>> print(format(relation))
            timespantools.OffsetTimespanTimeRelation(
                inequality=timespantools.CompoundInequality(
                    [
                        timespantools.Inequality('offset < timespan.start'),
                        ],
                    logical_operator='and',
                    ),
                )

    ..  container:: example

        **Example 2.** Makes time relation indicating that offset ``1/2``
        happens before `timespan` starts:

        ::

            >>> offset = durationtools.Offset(1, 2)

        ::

            >>> relation = \
            ...     timespantools.offset_happens_before_timespan_starts(
            ...     offset=offset)

        ::

            >>> print(format(relation))
            timespantools.OffsetTimespanTimeRelation(
                inequality=timespantools.CompoundInequality(
                    [
                        timespantools.Inequality('offset < timespan.start'),
                        ],
                    logical_operator='and',
                    ),
                offset=durationtools.Offset(1, 2),
                )

    ..  container:: example

        **Example 3.** Makes time relation indicating that `offset` happens
        before timespan ``[2, 8)`` starts:

        ::

            >>> timespan = timespantools.Timespan(2, 8)

        ::

            >>> relation = \
            ...     timespantools.offset_happens_before_timespan_starts(
            ...     timespan=timespan)

        ::

            >>> print(format(relation))
            timespantools.OffsetTimespanTimeRelation(
                inequality=timespantools.CompoundInequality(
                    [
                        timespantools.Inequality('offset < timespan.start'),
                        ],
                    logical_operator='and',
                    ),
                timespan=timespantools.Timespan(
                    start_offset=durationtools.Offset(2, 1),
                    stop_offset=durationtools.Offset(8, 1),
                    ),
                )

    ..  container:: example

        **Example 4.** Makes time relation indicating that offset ``1/2``
        happens before timespan ``[2, 8)`` starts:

        ::

            >>> relation = timespantools.offset_happens_before_timespan_starts(
            ...     timespan=timespan,
            ...     offset=offset,
            ...     hold=True,
            ...     )

        ::

            >>> print(format(relation))
            timespantools.OffsetTimespanTimeRelation(
                inequality=timespantools.CompoundInequality(
                    [
                        timespantools.Inequality('offset < timespan.start'),
                        ],
                    logical_operator='and',
                    ),
                timespan=timespantools.Timespan(
                    start_offset=durationtools.Offset(2, 1),
                    stop_offset=durationtools.Offset(8, 1),
                    ),
                offset=durationtools.Offset(1, 2),
                )

    ..  container:: example

        **Example 5.** Evaluates time relation indicating that offset ``1/2``
        happens before timespan ``[2, 8)`` starts:

        ::

            >>> timespantools.offset_happens_before_timespan_starts(
            ...     timespan=timespan,
            ...     offset=offset,
            ...     hold=False,
            ...     )
            True

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'offset < timespan.start',
        ])

    time_relation = timespantools.OffsetTimespanTimeRelation(
        inequality,
        timespan=timespan,
        offset=offset)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-


def select_nonlast_logical_ties_in_pitched_runs(argument=None):
    r'''Selects nonlast logical ties in pitched runs.

    ..  container:: example

        ::

            >>> selector = selectortools.select_nonlast_logical_ties_in_pitched_runs()
            >>> f(selector)
            selectortools.Selector(
                callbacks=(
                    selectortools.PrototypeSelectorCallback(
                        prototype=scoretools.Leaf,
                        ),
                    selectortools.RunSelectorCallback(
                        prototype=(
                            scoretools.Note,
                            scoretools.Chord,
                            ),
                        ),
                    selectortools.LogicalTieSelectorCallback(
                        flatten=False,
                        pitched=False,
                        trivial=True,
                        ),
                    selectortools.SliceSelectorCallback(
                        stop=-1,
                        apply_to_each=True,
                        ),
                    selectortools.FlattenSelectorCallback(
                        depth=1,
                        ),
                    ),
                )

        ::

            >>> staff = Staff("c' d' ~ d' e' r f' g' r a' b' ~ b' c''")
            >>> tuplet = Tuplet((2, 3), staff[2:5])
            >>> tuplet = Tuplet((2, 3), staff[5:8])
            >>> result = selector(staff)
            >>> label(result).color_alternating()
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    \once \override Accidental.color = #red
                    \once \override Beam.color = #red
                    \once \override Dots.color = #red
                    \once \override NoteHead.color = #red
                    \once \override Stem.color = #red
                    c'4
                    \once \override Accidental.color = #blue
                    \once \override Beam.color = #blue
                    \once \override Dots.color = #blue
                    \once \override NoteHead.color = #blue
                    \once \override Stem.color = #blue
                    d'4 ~
                    \times 2/3 {
                        \once \override Accidental.color = #blue
                        \once \override Beam.color = #blue
                        \once \override Dots.color = #blue
                        \once \override NoteHead.color = #blue
                        \once \override Stem.color = #blue
                        d'4
                        e'4
                        r4
                    }
                    \once \override Accidental.color = #red
                    \once \override Beam.color = #red
                    \once \override Dots.color = #red
                    \once \override NoteHead.color = #red
                    \once \override Stem.color = #red
                    f'4
                    g'4
                    \times 2/3 {
                        r4
                        \once \override Accidental.color = #blue
                        \once \override Beam.color = #blue
                        \once \override Dots.color = #blue
                        \once \override NoteHead.color = #blue
                        \once \override Stem.color = #blue
                        a'4
                        \once \override Accidental.color = #red
                        \once \override Beam.color = #red
                        \once \override Dots.color = #red
                        \once \override NoteHead.color = #red
                        \once \override Stem.color = #red
                        b'4 ~
                    }
                    \once \override Accidental.color = #red
                    \once \override Beam.color = #red
                    \once \override Dots.color = #red
                    \once \override NoteHead.color = #red
                    \once \override Stem.color = #red
                    b'4
                    c''4
                }

        ::

            >>> for x in selector(staff):
            ...     x
            ...
            LogicalTie([Note("c'4")])
            LogicalTie([Note("d'4"), Note("d'4")])
            LogicalTie([Note("f'4")])
            LogicalTie([Note("a'4")])
            LogicalTie([Note("b'4"), Note("b'4")])

    '''
    from abjad.tools import selectortools
    selector = selectortools.select_pitched_runs()
    selector = selector.by_logical_tie(flatten=False)
    selector = selector[:-1]
    selector = selector.flatten(depth=1)
    if argument is None:
        return selector
    return selector(argument)
